﻿// Copyright (C) Microsoft Corporation. All Rights Reserved.
// This code released under the terms of the Microsoft Public License
// (Ms-PL, http://opensource.org/licenses/ms-pl.html).

using System;
using System.Diagnostics.CodeAnalysis;
using System.Globalization;
using System.IO;
using System.Runtime.Serialization;
using System.Speech.Synthesis;
using System.Windows.Forms;
using Delay.Properties;

namespace Delay
{
    public static class SayIt
    {
        [STAThread]
        [SuppressMessage("Microsoft.Maintainability", "CA1502:AvoidExcessiveComplexity", Justification = "Simple, easy to understand flow.")]
        public static void Main(string[] args)
        {
            try
            {
                // Detect bad syntax
                if ((null == args) || (0 == args.Length))
                {
                    throw new SayItException(Resources.ErrorSyntax);
                }

                // Set defaults
                VoiceGender gender = VoiceGender.NotSet;
                PromptVolume volume = PromptVolume.NotSet;
                PromptRate rate = PromptRate.NotSet;
                var haveVoiceAndStyle = false;

                // Process arguments
                var prompt = new PromptBuilder(CultureInfo.CurrentCulture);
                var i = 0;
                while (i < args.Length)
                {
                    // Capture argument
                    var arg = args[i];
                    i++;

                    // Ensure second part of argument is present (if required)
                    var param = "";
                    if (arg.StartsWith("--", StringComparison.OrdinalIgnoreCase))
                    {
                        if (i == args.Length)
                        {
                            throw new SayItException(string.Format(CultureInfo.CurrentCulture, Resources.ErrorMissingParameter, arg));
                        }
                        param = args[i];
                        i++;
                    }

                    if (ComparesEqual("--gender", arg))
                    {
                        // Parse "--gender" argument
                        ClearVoiceAndStyle(prompt, gender, volume, rate, ref haveVoiceAndStyle);
                        if (ComparesEqual("male", param))
                        {
                            gender = VoiceGender.Male;
                        }
                        else if (ComparesEqual("female", param))
                        {
                            gender = VoiceGender.Female;
                        }
                        else if (ComparesEqual("neutral", param))
                        {
                            gender = VoiceGender.Neutral;
                        }
                        else
                        {
                            throw new SayItException(string.Format(CultureInfo.CurrentCulture, Resources.ErrorBadParameter, arg, param));
                        }
                    }
                    else if (ComparesEqual("--volume", arg))
                    {
                        // Parse "--volume" argument
                        ClearVoiceAndStyle(prompt, gender, volume, rate, ref haveVoiceAndStyle);
                        if (ComparesEqual("silent", param))
                        {
                            volume = PromptVolume.Silent;
                        }
                        else if (ComparesEqual("extrasoft", param))
                        {
                            volume = PromptVolume.ExtraSoft;
                        }
                        else if (ComparesEqual("soft", param))
                        {
                            volume = PromptVolume.Soft;
                        }
                        else if (ComparesEqual("medium", param))
                        {
                            volume = PromptVolume.Medium;
                        }
                        else if (ComparesEqual("loud", param))
                        {
                            volume = PromptVolume.Loud;
                        }
                        else if (ComparesEqual("extraloud", param))
                        {
                            volume = PromptVolume.ExtraLoud;
                        }
                        else if (ComparesEqual("default", param))
                        {
                            volume = PromptVolume.Default;
                        }
                        else
                        {
                            throw new SayItException(string.Format(CultureInfo.CurrentCulture, Resources.ErrorBadParameter, arg, param));
                        }
                    }
                    else if (ComparesEqual("--rate", arg))
                    {
                        // Parse "--rate" argument
                        ClearVoiceAndStyle(prompt, gender, volume, rate, ref haveVoiceAndStyle);
                        if (ComparesEqual("extraslow", param))
                        {
                            rate = PromptRate.ExtraSlow;
                        }
                        else if (ComparesEqual("slow", param))
                        {
                            rate = PromptRate.Slow;
                        }
                        else if (ComparesEqual("medium", param))
                        {
                            rate = PromptRate.Medium;
                        }
                        else if (ComparesEqual("fast", param))
                        {
                            rate = PromptRate.Fast;
                        }
                        else if (ComparesEqual("extrafast", param))
                        {
                            rate = PromptRate.ExtraFast;
                        }
                        else
                        {
                            throw new SayItException(string.Format(CultureInfo.CurrentCulture, Resources.ErrorBadParameter, arg, param));
                        }
                    }
                    else if (ComparesEqual("--text", arg))
                    {
                        // Process text from file
                        SetVoiceAndStyle(prompt, gender, volume, rate, ref haveVoiceAndStyle);
                        prompt.AppendText(File.ReadAllText(param));
                        prompt.AppendText(" ");
                    }
                    else if (ComparesEqual("--ssml", arg))
                    {
                        // Process SSML from file
                        SetVoiceAndStyle(prompt, gender, volume, rate, ref haveVoiceAndStyle);
                        prompt.AppendSsml(param);
                        prompt.AppendText(" ");
                    }
                    else
                    {
                        // Process text from command-line
                        SetVoiceAndStyle(prompt, gender, volume, rate, ref haveVoiceAndStyle);
                        prompt.AppendText(arg);
                        prompt.AppendText(" ");
                    }
                }

                // Finalize and speak
                ClearVoiceAndStyle(prompt, gender, volume, rate, ref haveVoiceAndStyle);
                using (var synth = new SpeechSynthesizer())
                {
                    synth.Speak(prompt);
                }
            }
            catch (Exception e)
            {
                // Show exception and syntax
                MessageBox.Show(
                    e.Message + Resources.Syntax,
                    Resources.SayIt,
                    MessageBoxButtons.OK,
                    MessageBoxIcon.Information,
                    MessageBoxDefaultButton.Button1,
                    0);
            }
        }

        private static void SetVoiceAndStyle(PromptBuilder prompt, VoiceGender gender, PromptVolume volume, PromptRate rate, ref bool haveVoiceAndStyle)
        {
            if (!haveVoiceAndStyle)
            {
                // Open voice and style tags
                if (VoiceGender.NotSet != gender)
                {
                    prompt.StartVoice(gender);
                }
                if ((PromptVolume.NotSet != volume) || (PromptRate.NotSet != rate))
                {
                    prompt.StartStyle(new PromptStyle() { Volume = volume, Rate = rate });
                }
                haveVoiceAndStyle = true;
            }
        }

        private static void ClearVoiceAndStyle(PromptBuilder prompt, VoiceGender gender, PromptVolume volume, PromptRate rate, ref bool haveVoiceAndStyle)
        {
            if (haveVoiceAndStyle)
            {
                // Close voice and style tags
                if ((PromptVolume.NotSet != volume) || (PromptRate.NotSet != rate))
                {
                    prompt.EndStyle();
                }
                if (VoiceGender.NotSet != gender)
                {
                    prompt.EndVoice();
                }
                haveVoiceAndStyle = false;
            }
        }

        private static bool ComparesEqual(string strA, string strB)
        {
            return 0 == string.Compare(strA, strB, StringComparison.OrdinalIgnoreCase);
        }
    }

    [Serializable]
    public class SayItException : Exception
    {
        public SayItException() : base() { }
        public SayItException(string message) : base(message) { }
        public SayItException(string message, Exception innerException) : base(message, innerException) { }
        protected SayItException(SerializationInfo info, StreamingContext context) : base(info, context) { }
    }
}
